#include "ShaderConstants.fxh"
#include "setHSPE/struct.fxh"

/*
Please permission first. if you used this code:)
Twitter: @DevHSPE
Facebook: Wanzekk
Youtube: Wanze'
*/

struct VS_Input
{
    float3 position : POSITION;
    float4 color : COLOR;
#ifdef INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif

};


struct PS_Input
{
    float4 position : SV_Position;
    float4 color : COLOR;
#ifdef GEOMETRY_INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	uint renTarget_id : SV_RenderTargetArrayIndex;
#endif
};

static const float fogNear = 0.9;
static const float3 inverseLightDirection = float3( 0.62, 0.78, 0.0 );
static const float ambient = 0.7;

ROOT_SIGNATURE
void main(in VS_Input VSInput, out PS_Input PSInput)
{

float4 fc = FOG_COLOR;
float4 cc = CURRENT_COLOR;
float2 fcc = FOG_CONTROL;
float distpos = clamp(VSInput.position.z / RENDER_DISTANCE -.7,.0,.5);

detection at;
at.day = pow(max(min(1.-fc.b*1.2,1.), .0), .5);
at.night = pow(max(min(1.-fc.r *1.5,1.),.0),1.2);
at.rain = (1.-pow(fcc.y, 11.));

float4 cday = float4(1.2,1.2,1.2,1.);
float4 cnight = float4(.2,.2,.2,-4.8);
float4 crain = float4(1.05,1.05,1.05,1.);
float4 cdusk = float4(1.,.9,.8,-3.);
float4 colorc = lerp(lerp(lerp(cday, cdusk, at.day), cnight, at.night),crain,at.rain);
float4 JillValentine = lerp(lerp(VSInput.color*cc,fc,distpos+.5),fc,colorc);

#ifdef INSTANCEDSTEREO
	int i = VSInput.instanceID;
	PSInput.position = mul( WORLDVIEWPROJ_STEREO[i], float4( VSInput.position, 1 ) );
	float3 worldPos = mul(WORLD_STEREO, float4(VSInput.position, 1));
#else
	PSInput.position = mul(WORLDVIEWPROJ, float4(VSInput.position, 1));
	float3 worldPos = mul(WORLD, float4(VSInput.position, 1));
#endif
#ifdef GEOMETRY_INSTANCEDSTEREO
	PSInput.instanceID = VSInput.instanceID;
#endif 
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	PSInput.renTarget_id = VSInput.instanceID;
#endif
    PSInput.color = JillValentine;

	float depth = length(worldPos) / RENDER_DISTANCE;

    float fog = max( depth - fogNear, 0.0 );

}