#include "ShaderConstants.fxh"
#include "util.fxh"

/*
Please permission first. if you used this code:)
Twitter: @DevHSPE
Facebook: Wanzekk
Youtube: Wanze'
*/

struct PS_Input
{
    float4 position : SV_Position;
    float3 poss : POSS;
    float2 uv : TEXCOORD_0_FB_MSAA;
};

struct PS_Output
{
    float4 color : SV_Target;
};

static const float2 m = float2(2.,6.);
static const float pi = 3.141592;
static const float2x2 m2 = float2x2(.8,.6,-.6,.8);

float ssnoise(in float2 p){
    float res=0.;
    float f=2.;
	for( int i=0; i< 4; i++ ){
        p= mul(m2,p*f+.6);
        f*=1.0;
        res+=sin(p.x+sin(2.*p.y));}
	return res/4.;}

float fbmabs(float2 p ) {
	float f=1.;
	float r = 0.0;
    for(int i = 0;i<8;i++){
		r += abs(ssnoise( p*f ))/f;
	    f *=2.;
        p-=float2(-.01,.08)*r;}
	return r;}

float fbmstars( float2 p ) {
    p=floor(p*50.)/50.;
	float f=1.;
	float r = 0.0;
    for(int i = 1;i<5;i++){
		r += ssnoise( p*(20.+3.*f) )/f;
        p = mul(p,m2);
	    f +=1.;}
	return pow(r,8.);}

float fbmdisk( float2 p ) {
	float f=1.;
	float r = 0.0;
    for(int i = 1;i<7;i++){
		r += abs(ssnoise( p*(f) ))/f;
	    f +=1.;}
	return 1./r;}

float fbmdust( float2 p ) {
float f=1.;
	float r = 0.0;
    for(int i = 1;i<7;i++){
		r += 1./abs(ssnoise( p*(f) ))/f;
	    f +=1.;}
	return pow(1.-1./r,4.);}

float theta(float r, float wb, float wn){
return atan(exp(1./r)/wb)*2.*wn;}

float arm(float n, float aw, float wb, float wn,float2 p){
    float t = atan2(p.y,p.x);
    float r = length(p);
	return pow(1.-.15*sin((theta(r,wb,wn)-t)*n),aw)*exp(-r*r)*exp(-.07/r);}

float2 maparm(float n, float aw, float wb, float wn,float2 p){ 
 float t = atan2(p.y,p.x);
   float r = length(p);
	return float2((theta(r,wb,wn)-t)*n,r);}

float bulb(float2 p){
    float r = exp(-dot(p,p)*1.2);
    p.y-=.2;
	return r+.5*exp(-dot(p,p)*12.);}

float map(float2 p){
    float a= arm(m.x,6.,.7,m.y,p);
    float d = fbmdust(p);
    float r = max(a*(.4+.1*arm(m.x+1.,4.,.7,m.y,mul(p,m2)))*(.1+.6*d+.4*fbmdisk(p)),bulb(p)*(.7+.2*d+.2*fbmabs(p)));
    return max(r, a*fbmstars(p*4.));}

float2 rotate(in float2 p, in float t){
	return p * cos(-t) + float2(p.y, -p.x) * sin(-t);}


ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{
#if !defined(TEXEL_AA) || !defined(TEXEL_AA_FEATURE) || (VERSION < 0xa000 /*D3D_FEATURE_LEVEL_10_0*/) 
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv);
#else
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv);
#endif

#ifdef ALPHA_TEST
    if( diffuse.a < 0.5 )
    {
        discard;
    }
#endif

int windows = 0;
float2 me = float2(m);
float2x2 matr = float2x2(m2);
float4 iMouse = float4(0.,0.,0.,0.);

  float2 peli = PSInput.poss.xz*2.5;
    peli *=2.;
	if(peli.y>0.){
    	if(peli.x>0.)windows =1;
    	else    windows =0;}
    else{
    	if(peli.x>0.)windows =3;
        else windows =2;}
       peli = rotate(peli,-.1*TIME);
    if(iMouse.z>0.)me = floor(iMouse.xy/PSInput.poss.xy*10.);
    me.y*=2.;
	float r;

float3 light = normalize(float3(4., 2., -1.));
    float k=1.5*map(peli);
    float b=.3*map(mul(peli,matr))+.4;
    r=.2;

diffuse = clamp(float4(r*k*k, r*k, k*.5+b*.4, 1.0),0.,1.)*8.8;
                   
#ifdef IGNORE_CURRENTCOLOR
    PSOutput.color = diffuse;
#else
    PSOutput.color = CURRENT_COLOR * diffuse;
#endif

#ifdef WINDOWSMR_MAGICALPHA
    // Set the magic MR value alpha value so that this content pops over layers
    PSOutput.color.a = 133.0f / 255.0f;
#endif
}
