#include "ShaderConstants.fxh"
#include "util.fxh"
#include "setHSPE/struct.fxh"
#include "setHSPE/detection.fxh"
#include "setHSPE/tonemap.fxh"
#include "setHSPE/mapping.fxh"
#include "setHSPE/experimental.fxh"

/*
Please permission first. if you used this code:)
Twitter: @DevHSPE
Facebook: Wanzekk
Youtube: Wanze'
*/

struct PS_Input
{
	float4 position : SV_Position;
        float3 poss : POSS;
        float3 wpos : WPOS;
        float3 screen : SCREEN;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
};

struct PS_Output
{
	float4 color : SV_Target;
};


ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput)
{
#ifdef BYPASS_PIXEL_SHADER
    PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
    return;
#else

#if USE_TEXEL_AA
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv0 );
#else
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
#endif

#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
#endif

float4 inColor = PSInput.color;
float4 fc = FOG_COLOR;
float3 fv = sqrt(1.3*inColor.rgb);
float2 fcc = FOG_CONTROL;
float shy = PSInput.uv1.y;
float shx = PSInput.uv1.x;
float4 wnz = float4(PSInput.uv0,PSInput.uv0);
float4 tex = TEXTURE_0.Sample(TextureSampler0,PSInput.uv0);
float4 timeset = TEXTURE_1.Sample(TextureSampler1, float2(0.0,1.0));

detection at;
at.day = clamp((timeset.r-.5)*2.,.0,1.);
at.dusk= 2.*(.5-abs(.5-clamp((at.day-.4)*3.,0.,1.)));
at.night = 1.0-at.day;
at.rain = (1.0-pow(fcc.y,5.0))*1.0;
at.morning = pow(max(min(1.0-fc.b*1.2,1.0),0.0),0.5);
at.midnight = pow(max(min(1.0-fc.r*1.5,1.0),0.0),1.2);
at.cave = 1.0-shy;
at.sunset = clamp((1. - fc.b)*1.5, .0, 1.)*clamp((fc.r -.15)*1.25,0.,1.);

at.day *= shy;
at.night *= shy;
at.rain *= shy;
at.dusk *= shy;
at.morning *= shy;
at.midnight *= shy;
at.sunset *= shy;

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
		#define ALPHA_THRESHOLD 0.05
	#else
		#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif

#if defined(BLEND)
	diffuse.a *= PSInput.color.a;
#endif

#if !defined(ALWAYS_LIT)
	diffuse = diffuse * TEXTURE_1.Sample(TextureSampler1, PSInput.uv1);
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = PSInput.color.a;
	#endif	

	diffuse.rgb *= lerp(inColor.rgb, fv, at.cave);
#else
	float2 uv = PSInput.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uv).rgb*2.0f, PSInput.color.b);
	diffuse.rgb *= PSInput.color.aaa;
	diffuse.a = 1.0f;
#endif

float3 dirlight1 = normalize(cross(ddx(PSInput.wpos),ddy(PSInput.wpos)));
float3 dirlight2 = normalize(cross(ddx(PSInput.poss), ddy(PSInput.poss)));
float3 dirlight3 = normalize(cross(ddx(PSInput.poss.zyx), ddy(PSInput.poss.zyx)));

other clr;
clr.Shadows = float3(.925,.925,.925)+shx*float3(.12,.12,.12);
clr.Shadows = lerp(clr.Shadows, float3(1.,1.,1.), at.night);
clr.Shadows = lerp(clr.Shadows, float3(1.,1.,1.), at.rain);
clr.Shadows = lerp(clr.Shadows, float3(.95,.95,.95), at.dusk);
clr.Leaves = float3(1.1,1.2,1.1);
clr.Exposure = float3(1.2,1.2,1.2);
clr.underWater = float3(1.,1.,1.5);
clr.SeaLantern = float3(0., 2.8, 3.);
clr.Grass = float3(.8,.8,.8);
clr.Distpos = inColor.a *(length(PSInput.wpos) / RENDER_DISTANCE * +.7);

float shside = PSInput.color.g+0.7*(PSInput.color.g-PSInput.color.b);
float3 Torch = light(diffuse.rgb, clr, shx, at);
float3 clrDirlight1 = lerp(float3(1.,1.,1.), float3(1.2,1.2,1.2), max(0.0, dirlight2.z));
float3 clrDirlight2 = lerp(float3(1.,1.,1.), float3(1.2,1.2,1.2), max(0.0, dirlight3.x));
float3 clrDirlight3 = lerp(float3(1.,1.,1.), float3(1.5,1.5,.0), max(0.0, dirlight2.z));
float3 clrDirlight4 = lerp(float3(1.,1.,1.), float3(1.5,1.5,.0), max(0.0, dirlight3.x));
float4 wet = TEXTURE_0.Sample(TextureSampler0,diffuse.xz);
float ref_b = refleksi(PSInput.wpos,dirlight1,at);
float3 glow1 = lerp(float3(1.,1.,1.), float3(2.6,2.6,2.6), at.night);
float3 glow2 = lerp(float3(1.,1.,1.), float3(10.,10.,10.), at.cave);
float noglow = (1.-shx);

diffuse.rgb = HSTM(diffuse.rgb);
diffuse.rgb = HSS(diffuse.rgb, 1.7);
diffuse.rgb *= clrWorld(diffuse.rgb, at);
diffuse.rgb *= lerp(float3(1.,1.,1.),HSS(diffuse.rgb, .4), at.rain);
diffuse.rgb *= clrDirlight1;
diffuse.rgb *= lerp(float3(1.,1.,1.), HSS(diffuse.rgb, .6), at.rain);
diffuse.rgb = lerp(diffuse.rgb,HSS(diffuse.rgb, .8), at.night);

if(!isWater(PSInput.color) && !isUnderwater(fcc, fc)){
diffuse.rgb += puddle(PSInput.poss, at)*max(0.,-dirlight1.y)*5.*diffuse.rgb;
diffuse.rgb += wet_wave(PSInput.poss)*.7*at.rain*(1.-at.cave)*(1.-at.night)*max(0.,-dirlight1.y)*diffuse.rgb;
diffuse += wet*.28*at.rain*(1.-at.night);}

if(PSInput.color.a == 0.0){
diffuse.rgb *= clr.Leaves;
shside = PSInput.color.g*1.2;
clr.Shadows = float3(.89,.89,.89)+shx*float3(.16,.16,.16);}
diffuse.rgb = shadow(diffuse.rgb, clr, shside, shy);
if(PSInput.color.g > PSInput.color.r){diffuse.rgb *= clr.Grass;}
if(!(fcc.x < .55)){
diffuse.rgb = sunbeam(diffuse.rgb, PSInput.screen, PSInput.wpos,PSInput.poss, at);
diffuse.rgb *= lerp(float3(1.,1.,1.), clrDirlight4, at.dusk);
if(!(PSInput.color.g>PSInput.color.r&&PSInput.color.a > 0.)){
diffuse.rgb *= clrDirlight2;
diffuse.rgb *= lerp(float3(1.,1.,1.), clrDirlight3, at.dusk);}}

#ifndef SEASONS
if(isWater(PSInput.color)){
diffuse.rgb = clrWater(diffuse.rgb, at);
diffuse.a -= .15;
diffuse.a += clr.Distpos;}

if(!isUnderwater(fcc, fc)){
diffuse.rgb += Torch;}else{
diffuse.rgb += clr.SeaLantern*pow(shx*.3,1.5);
diffuse.rgb *= clr.underWater;}
#endif

#ifdef  BLEND
if(!(.8 > shy)){diffuse.rgb += ref_b;}
#endif

if(isGold(wnz) || isIron(wnz)){
diffuse.rgb += ref_b*1.2;}
if(isDiamond(wnz) || isRedstone(wnz)){
diffuse.rgb += ref_b*1.1;} 
if(isLapisLazuli(wnz)){ 
diffuse.rgb += ref_b*1.3;}
if(isEmerald(wnz)){
diffuse.rgb += ref_b*1.15;}

if(isDiamondOre(wnz) || isEmeraldOre(wnz)){
if(!isStoneOres(tex)){
diffuse.rgb *= glow1*1.2*noglow;
diffuse.rgb *= glow2*1.5*noglow;}}
if(isRedstoneOre(wnz) || isLapisOre(wnz)){
if(!isStoneOres(tex)){
diffuse.rgb *= glow1*1.5*noglow;
diffuse.rgb *= glow2*2.*noglow;}}
if(isGoldOre(wnz) || isIronOre(wnz)){
if(!isStoneOres(tex)){
diffuse.rgb *= glow1*noglow;
diffuse.rgb *= glow2*1.5*noglow;}}

if(isNether(fc)){
Torch *= .2;
diffuse.rgb *= float3(1.2,1.2,1.2);
float FogNthr = clamp(length(PSInput.wpos)/RENDER_DISTANCE* 1.,.3,2.);
float3 FogNthrON = lerp(diffuse.rgb, fc.rgb*float3(1.6,.5,.1),FogNthr);
diffuse.rgb = FogNthrON;}

if(isTheEnd(fc)){
Torch *= 0.;
float3 TorchEnd = float3(.2,.0,2.)*pow(shx*1.,2.);
diffuse.rgb += TorchEnd;}

#ifdef LOW_PRECISION
FOGGY fog;
fog.Atmosphere = length(-PSInput.wpos.xz *2.) / FAR_CHUNKS_DISTANCE*.2;
fog.Atmosphere = clamp(fog.Atmosphere, .1, .8);
diffuse.rgb = lerp(diffuse.rgb, fc.rgb*float3(1.,1.1,1.1), fog.Atmosphere);

fog.foggyRain = clamp(length(-PSInput.wpos) / RENDER_DISTANCE *1.5,.0,1.);
fog.foggyRainON = lerp(diffuse.rgb, fc.rgb*float3(2.,2.,2.), fog.foggyRain);
diffuse.rgb = lerp(diffuse.rgb, fog.foggyRainON, at.rain);

fog.foggyDusk = clamp(length(PSInput.wpos) / RENDER_DISTANCE *1.4, .15, 1.);
fog.foggyDuskON = lerp(diffuse.rgb, fc.rgb, fog.foggyDusk);
diffuse.rgb = lerp(diffuse.rgb, fog.foggyDuskON, at.morning);

fog.foggyNight = clamp(length(PSInput.wpos) / FAR_CHUNKS_DISTANCE * .1, .4, 1.5);
fog.foggyNightON = lerp(diffuse.rgb, fc.rgb, fog.foggyNight);
diffuse.rgb = lerp(diffuse.rgb, fog.foggyNightON, at.midnight);
#endif

float fogground = clamp(length(PSInput.wpos.y*2.)/RENDER_DISTANCE*3.3,.1,1.2);
float3 groundfog = lerp(diffuse.rgb, fc.rgb, fogground);
diffuse.rgb = lerp(diffuse.rgb, groundfog, at.morning); 

#ifdef FOG
	diffuse.rgb = lerp( diffuse.rgb, PSInput.fogColor.rgb, PSInput.fogColor.a );
#endif

	PSOutput.color = diffuse;

#ifdef VR_MODE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to 
	// the lowest 8 bit value.
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif

#endif // BYPASS_PIXEL_SHADER
}