#include "ShaderConstants.fxh"
#include "setHSPE/detection.fxh"

/*
Please permission first. if you used this code:)
Twitter: @DevHSPE
Facebook: Wanzekk
Youtube: Wanze'
*/

struct VS_Input {
	float3 position : POSITION;
	float4 color : COLOR;
	float2 uv0 : TEXCOORD_0;
	float2 uv1 : TEXCOORD_1;
#ifdef INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
};


struct PS_Input {
	float4 position : SV_Position;
        float3 poss : POSS;
        float3 wpos : WPOS;
        float3 screen : SCREEN;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
#ifdef GEOMETRY_INSTANCEDSTEREO
	uint instanceID : SV_InstanceID;
#endif
#ifdef VERTEXSHADER_INSTANCEDSTEREO
	uint renTarget_id : SV_RenderTargetArrayIndex;
#endif
};


static const float rA = 1.0;
static const float rB = 1.0;
static const float3 UNIT_Y = float3(0, 1, 0);
static const float DIST_DESATURATION = 56.0 / 255.0; //WARNING this value is also hardcoded in the water color, don'tchange


ROOT_SIGNATURE
void main(in VS_Input VSInput, out PS_Input PSInput)
{
#ifndef BYPASS_PIXEL_SHADER
	PSInput.uv0 = VSInput.uv0;
	PSInput.uv1 = VSInput.uv1;
	PSInput.color = VSInput.color;
#endif

#ifdef AS_ENTITY_RENDERER
	#ifdef INSTANCEDSTEREO
		int i = VSInput.instanceID;
		PSInput.position = mul(WORLDVIEWPROJ_STEREO[i], float4(VSInput.position, 1));
	#else
		PSInput.position = mul(WORLDVIEWPROJ, float4(VSInput.position, 1));
	#endif
		float3 worldPos = PSInput.position;
#else
		float3 worldPos = (VSInput.position.xyz * CHUNK_ORIGIN_AND_SCALE.w) + CHUNK_ORIGIN_AND_SCALE.xyz;
	
		// Transform to view space before projection instead of all at once to avoid floating point errors
		// Not required for entities because they are already offset by camera translation before rendering
		// World position here is calculated above and can get huge
	#ifdef INSTANCEDSTEREO
		int i = VSInput.instanceID;
	
		PSInput.position = mul(WORLDVIEW_STEREO[i], float4(worldPos, 1 ));
		PSInput.position = mul(PROJ_STEREO[i], PSInput.position);
	
	#else
		PSInput.position = mul(WORLDVIEW, float4( worldPos, 1 ));
		PSInput.position = mul(PROJ, PSInput.position);
	#endif

#endif

               PSInput.wpos = worldPos.xyz;
               PSInput.poss = VSInput.position.xyz;
               VSInput.uv1.x = max((VSInput.uv1.x - .32) * 1.5, .0);
               PSInput.screen.xy = PSInput.position.xy / (PSInput.position.z + 1.0);


#ifdef GEOMETRY_INSTANCEDSTEREO
		PSInput.instanceID = VSInput.instanceID;
#endif 
#ifdef VERTEXSHADER_INSTANCEDSTEREO
		PSInput.renTarget_id = VSInput.instanceID;
#endif
///// find distance from the camera

#if defined(FOG) || defined(BLEND)
	#ifdef FANCY
		float3 relPos = -worldPos;
		float cameraDepth = length(relPos);
	#else
		float cameraDepth = PSInput.position.z;
	#endif
#endif

float3 wave = worldPos.xyz;
float3 waves = VSInput.position.xyz;

float rain = (1.-pow(FOG_CONTROL.y, 11.)) * VSInput.uv1.y;
float pq = sin(TIME*2.3+wave.x+cos(TIME*1.4+wave.x+wave.y+wave.z));
float obh = .09*sin(2.0 * (TIME + waves.x  + waves.z))+ .05 * sin(2.* (TIME*1.2 + waves.x )); 
float obh_b = .09*sin(2.0 * (TIME + waves.x  + waves.x + waves.z + waves.y + waves.x))+ .05 * sin(2.* (TIME*1.2 + waves.z ));

#ifdef ALPHA_TEST
if(PSInput.color.g>PSInput.color.r&&PSInput.color.a>0.|| PSInput.color.r > PSInput.color.g && PSInput.color.g > PSInput.color.b && PSInput.color.a != 0.){
PSInput.position.x += sin(TIME*.5+waves.z+waves.z+waves.z+cos(TIME*1.5+waves.y+waves.x))*sin(waves.x+waves.y)*.052;} 

if(PSInput.color.a == 0.){
PSInput.position.x += sin(TIME * 3. + waves.x + waves.z + waves.x + waves.z + waves.y) * sin(waves.z) * .04;}

if(PSInput.color.g + PSInput.color.g > PSInput.color.r + PSInput.color.b){
PSInput.position.x += sin(TIME * 5.5 + waves.x + waves.y + waves.z) * .04 * rain;}
#endif

#ifndef SEASONS
if(isWater(PSInput.color)){
PSInput.position.y += pq*.085 ;}
if(isWater(PSInput.color)){
PSInput.position.y += pq*.02 ;}
if(isWater(PSInput.color)){
PSInput.position.y += pq*.01 ;}

if(isUnderwater(FOG_CONTROL, FOG_COLOR) && PSInput.color.a != 0.){
PSInput.color.rgb += obh-obh_b+obh*obh_b*float3(.1,.1,.1);
PSInput.position.xy += sin(TIME * 2.8 + wave.x + wave.z + wave.y + wave.z ) * .025;}
#endif




	///// apply fog

#ifdef FOG
	float len = cameraDepth / RENDER_DISTANCE;
#ifdef ALLOW_FADE
	len += RENDER_CHUNK_FOG_ALPHA.r;
#endif

	PSInput.fogColor.rgb = FOG_COLOR.rgb;
	PSInput.fogColor.a = clamp((len - FOG_CONTROL.x) / (FOG_CONTROL.y - FOG_CONTROL.x), 0.0, 1.0);

#endif

///// blended layer (mostly water) magic
#ifdef BLEND
	//Mega hack: only things that become opaque are allowed to have vertex-driven transparency in the Blended layer...
	//to fix this we'd need to find more space for a flag in the vertex format. color.a is the only unused part
	bool shouldBecomeOpaqueInTheDistance = VSInput.color.a < 0.95;
	if(shouldBecomeOpaqueInTheDistance) {
		#ifdef FANCY  /////enhance water
			float cameraDist = cameraDepth / FAR_CHUNKS_DISTANCE;
		#else
			float3 relPos = -worldPos.xyz;
			float camDist = length(relPos);
			float cameraDist = camDist / FAR_CHUNKS_DISTANCE;
		#endif //FANCY
		
		float alphaFadeOut = clamp(cameraDist, 0.0, 1.0);
		PSInput.color.a = lerp(VSInput.color.a, 1.0, alphaFadeOut);
	}
#endif

}
